/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { LocalOrderer } from "@fluidframework/server-memory-orderer";
import { GitManager } from "@fluidframework/server-services-client";
import { CheckpointService, MongoCheckpointRepository, MongoDocumentRepository, } from "@fluidframework/server-services-core";
export class LocalOrdererManager {
    constructor(storage, databaseManager, createHistorian, logger, serviceConfiguration, pubsub, documentRepository, checkpointRepository) {
        this.storage = storage;
        this.databaseManager = databaseManager;
        this.createHistorian = createHistorian;
        this.logger = logger;
        this.serviceConfiguration = serviceConfiguration;
        this.pubsub = pubsub;
        this.documentRepository = documentRepository;
        this.checkpointRepository = checkpointRepository;
        /**
         * Map of "tenantId/documentId" to the orderer for that document.
         */
        this.ordererMap = new Map();
    }
    /**
     * Closes all local orderers
     */
    async close() {
        await Promise.all(Array.from(this.ordererMap.values()).map(async (orderer) => (await orderer).close()));
        this.ordererMap.clear();
    }
    /**
     * Returns true if there are any received ops that are not yet ordered.
     */
    async hasPendingWork() {
        return Promise.all(this.ordererMap.values()).then((orderers) => {
            for (const orderer of orderers) {
                // We know that it ia LocalOrderer, break the abstraction
                if (orderer.hasPendingWork()) {
                    return true;
                }
            }
            return false;
        });
    }
    async getOrderer(tenantId, documentId) {
        const key = `${tenantId}/${documentId}`;
        let orderer = this.ordererMap.get(key);
        if (orderer === undefined) {
            orderer = this.createLocalOrderer(tenantId, documentId);
            this.ordererMap.set(key, orderer);
        }
        return orderer;
    }
    async createLocalOrderer(tenantId, documentId) {
        var _a, _b, _c;
        const historian = await this.createHistorian(tenantId);
        const gitManager = new GitManager(historian);
        const documentRepository = (_a = this.documentRepository) !== null && _a !== void 0 ? _a : new MongoDocumentRepository(await this.databaseManager.getDocumentCollection());
        const deliCheckpointRepository = (_b = this.checkpointRepository) !== null && _b !== void 0 ? _b : new MongoCheckpointRepository(await this.databaseManager.getCheckpointCollection(), "deli");
        const scribeCheckpointRepository = (_c = this.checkpointRepository) !== null && _c !== void 0 ? _c : new MongoCheckpointRepository(await this.databaseManager.getCheckpointCollection(), "scribe");
        const deliCheckpointService = new CheckpointService(deliCheckpointRepository, documentRepository, false);
        const scribeCheckpointService = new CheckpointService(scribeCheckpointRepository, documentRepository, false);
        const orderer = await LocalOrderer.load(this.storage, this.databaseManager, tenantId, documentId, this.logger, documentRepository, deliCheckpointRepository, scribeCheckpointRepository, deliCheckpointService, scribeCheckpointService, gitManager, undefined /* ILocalOrdererSetup */, this.pubsub, undefined /* broadcasterContext */, undefined /* scriptoriumContext */, undefined /* scribeContext */, undefined /* deliContext */, undefined /* moiraContext */, this.serviceConfiguration);
        const lambdas = [
            orderer.broadcasterLambda,
            orderer.deliLambda,
            orderer.scribeLambda,
            orderer.scriptoriumLambda,
        ];
        await Promise.all(lambdas.map(async (lambda) => {
            if (lambda === undefined) {
                throw new Error("We expect lambdas to be defined by now.");
            }
            if (lambda.state === "created") {
                return new Promise((resolve) => lambda.once("started", () => resolve()));
            }
        }));
        return orderer;
    }
}
//# sourceMappingURL=localOrdererManager.js.map